package org.rcisoft.business.cmsbanner.service.impl;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.cmsbanner.dao.CmsBannerRepository;
import org.rcisoft.business.cmsbanner.entity.CmsBanner;
import org.rcisoft.business.cmsbanner.entity.CmsBannerDTO;
import org.rcisoft.business.cmsbanner.service.CmsBannerService;
import org.rcisoft.business.cmsbannerunit.dao.CmsBannerUnitRepository;
import org.rcisoft.business.cmsbannerunit.entity.CmsBannerUnit;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.component.CyInitComp;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyIdGenUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Created by cy on 2024年3月30日 下午1:51:15.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class CmsBannerServiceImpl extends ServiceImpl<CmsBannerRepository, CmsBanner> implements CmsBannerService {

    @Autowired
    private Global global;
    @Autowired
    private CyInitComp cyInitComp;
    @Autowired
    private CmsBannerUnitRepository cmsBannerUnitRepository;


    /**
     * 保存 banner管理
     *
     * @param cmsBanner
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(CmsBanner cmsBanner) {
        //增加操作
        int line = baseMapper.insert(cmsBanner);
        //如果传入了unitId在增加关联关系
        if (cmsBanner.getUnitId() != null && !"".equals(cmsBanner.getUnitId())) {
            CmsBannerUnit cmsBannerUnit = new CmsBannerUnit();
            cmsBannerUnit.setCmsBannerId(cmsBanner.getBusinessId());
            cmsBannerUnit.setSysUnitId(Integer.valueOf(cmsBanner.getUnitId()));
            cmsBannerUnitRepository.insert(cmsBannerUnit);
        }

        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                cmsBanner.getBusinessId() + "的banner管理信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 banner管理
     *
     * @param cmsBanner
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(CmsBanner cmsBanner) {
        int line = baseMapper.realDelete(cmsBanner);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                cmsBanner.getBusinessId() + "的banner管理信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 banner管理
     *
     * @param cmsBanner
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(CmsBanner cmsBanner) {
        //判断用户身份
        //超级管理员
        if (CyUserUtil.getAuthenRoleIds().equals(cyInitComp.admin[0])) {
            cmsBanner.setDeleted();
            //根据bannerId逻辑删除banner信息
            int line = baseMapper.deleteById(cmsBanner);
            //根据bannerId物理删除关联表信息（banner商家信息关联表）
            cmsBannerUnitRepository.deleteUnitByBannerId(cmsBanner.getBusinessId());
        }
        //商家管理员
        if (CyUserUtil.getAuthenRoleIds().equals(cyInitComp.unit[0])) {
            //根据bannerId物理删除关联表信息（banner商家信息关联表）
            cmsBannerUnitRepository.deleteUnitByBannerIdUnitid(cmsBanner.getBusinessId(), cmsBanner.getUnitId());
        }
        return new CyPersistModel(1);
    }

    /**
     * 修改 banner管理
     *
     * @param cmsBanner
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(CmsBanner cmsBanner) {
        int line = baseMapper.updateById(cmsBanner);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + cmsBanner.getBusinessId() + "的banner管理信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 banner管理
     *
     * @param id
     * @return
     */
    @Override
    public CmsBanner findById(String id) {
        CmsBanner banner = baseMapper.selectByIdWithUrl(id);
        if (banner.getUrl() != null) {
            banner.setUrl(global.getBase_Discovery() + banner.getUrl());
        }
        return banner;
    }

    /**
     * 分页查询 banner管理
     *
     * @param cmsBanner
     * @return
     */
    @Override
    public IPage<CmsBanner> findAllByPagination(CyPageInfo<CmsBanner> paginationUtility,
                                                CmsBanner cmsBanner) {
        IPage<CmsBanner> result = baseMapper.queryCmsBannersPaged(paginationUtility, cmsBanner);
        List<CmsBanner> cmsBannerList = result.getRecords();
        for (CmsBanner banner : cmsBannerList) {
            if (banner.getUrl() != null) {
                banner.setUrl(global.getBase_Discovery() + banner.getUrl());
            }
        }
        return result;

    }

    /**
     * 查询list banner管理
     *
     * @param cmsBanner
     * @return
     */
    @Override
    public List<CmsBanner> findAll(CmsBanner cmsBanner) {
        return baseMapper.queryCmsBanners(cmsBanner);

    }

    /**
     * 导出banner管理
     *
     * @return
     */
    @Override
    public List<CmsBanner> export(CmsBanner cmsBanner) {
        List<CmsBanner> cmsBannerList = baseMapper.queryCmsBanners(cmsBanner);
        return cmsBannerList;
    }

    @Override
    public IPage<CmsBanner> findAllByPaginationForWx(CyPageInfo<CmsBanner> paginationUtility, CmsBannerDTO cmsBanner) {
        //判断当前用户类型 1超级管理员 2商家管理员 3代言人
        String authenRoleIds = CyUserUtil.getAuthenRoleIds();
        //第四种 游客  用户必有角色 没有角色必为游客
        if (authenRoleIds == null) {
            paginationUtility.setSize(cmsBanner.getRows());
            paginationUtility.setCurrent(cmsBanner.getPage());
            IPage<CmsBanner> cmsBannerIPage = baseMapper.visitorPagination(paginationUtility, cmsBanner);
            //循环添加url头
            cmsBannerIPage.getRecords().forEach(banner -> {
                if (banner.getUrl() != null) {
                    banner.setUrl(global.getBase_Discovery() + banner.getUrl());
                }
            });
            return cmsBannerIPage;
        }
        //第一种 1超级管理员
        if (authenRoleIds.contains(cyInitComp.admin[0])) {
            //超级管理员可以看到全部的banner
            //分页
            paginationUtility.setSize(cmsBanner.getRows());
            paginationUtility.setCurrent(cmsBanner.getPage());
            IPage<CmsBanner> cmsBannerIPage = baseMapper.superAdminPaginationCheck(paginationUtility, cmsBanner);
            //循环添加url头
            cmsBannerIPage.getRecords().forEach(banner -> {
                if (banner.getUrl() != null) {
                    banner.setUrl(global.getBase_Discovery() + banner.getUrl());
                }
            });
            return cmsBannerIPage;
        }
        //第二种 2商家管理员 配置类中存在角色
        if (authenRoleIds.contains(cyInitComp.permission[0])) {
            //banner
            paginationUtility.setSize(cmsBanner.getRows());
            paginationUtility.setCurrent(cmsBanner.getPage());
            IPage<CmsBanner> cmsBannerIPage = baseMapper.merchantAdminPaginationCheck(paginationUtility, cmsBanner);
            //循环添加url头
            cmsBannerIPage.getRecords().forEach(banner -> {
                if (banner.getUrl() != null) {
                    banner.setUrl(global.getBase_Discovery() + banner.getUrl());
                }
            });
            return cmsBannerIPage;
        }
        //第三种 3代言人  配置类中存在角色
        if (authenRoleIds.contains(cyInitComp.permission[1])) {
            //当前代言人的用户id
            cmsBanner.setUserId(CyUserUtil.getAuthenBusinessId());
            //分页
            paginationUtility.setSize(cmsBanner.getRows());
            paginationUtility.setCurrent(cmsBanner.getPage());
            IPage<CmsBanner> cmsBannerIPage = baseMapper.spokespersonPagination(paginationUtility, cmsBanner);
            //循环添加url头
            cmsBannerIPage.getRecords().forEach(banner -> {
                if (banner.getUrl() != null) {
                    banner.setUrl(global.getBase_Discovery() + banner.getUrl());
                }
            });
            return cmsBannerIPage;
        }
        return null;
    }

    @Override
    public CyPersistModel updateCmsBannerById(CmsBanner cmsBanner) {
        //updateById 目前只会根据id 修改传入的参数值,参数为null的不会修改
        int line = baseMapper.updateById(cmsBanner);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + cmsBanner.getBusinessId() + "的banner管理信息");
        return new CyPersistModel(line);
    }

    @Override
    public CyPersistModel assignBannerToBusiness(CmsBanner cmsBanner) {
        List<String> collect = Arrays.stream(cmsBanner.getUnitIds().split(",")).collect(Collectors.toList());
        //先清除当前的联系 根据bannerid删除
        cmsBannerUnitRepository.deleteUnitByBannerId(cmsBanner.getBusinessId());
        //创建对象
        if (collect.size() >= 0 && !"".equals(cmsBanner.getUnitIds())) {
//            List<CmsBannerUnit> cmsBannerUnits = new ArrayList<>();
            for (String s : collect) {
                CmsBannerUnit cmsBannerUnit = new CmsBannerUnit();
                cmsBannerUnit.setBusinessId(CyIdGenUtil.uuid());
                cmsBannerUnit.setCmsBannerId(cmsBanner.getBusinessId());
                cmsBannerUnit.setSysUnitId(Integer.parseInt(s));
                cmsBannerUnitRepository.insert(cmsBannerUnit);
//                CmsBannerUnit apply = cmsBannerUnit;
//                cmsBannerUnits.add(apply);
            }
//            int line = cmsBannerUnitRepository.assignBannerToBusiness(cmsBannerUnits);

        }
        return new CyPersistModel(1);

    }

}
