package org.rcisoft.business.cmsbannerunit.controller;


/*固定导入*/

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.cmsbannerunit.entity.CmsBannerUnit;
import org.rcisoft.business.cmsbannerunit.service.CmsBannerUnitService;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyResultGenUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * Created by cy on 2024年6月7日 下午2:29:37.
 */
@RestController
@RequestMapping("/cmsbannerunit")
public class CmsBannerUnitController extends CyPaginationController<CmsBannerUnit> {


    @Autowired
    private CmsBannerUnitService cmsBannerUnitServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:bannerUnit:add')")
    @CyOpeLogAnno(title = "system-banner与商家中间表管理-新增banner与商家中间表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value = "添加banner与商家中间表", notes = "添加banner与商家中间表")
    @PostMapping(value = "/add")
    public CyResult add(@Valid CmsBannerUnit cmsBannerUnit, BindingResult bindingResult) {
        CyPersistModel data = cmsBannerUnitServiceImpl.persist(cmsBannerUnit);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsBannerUnit);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:bannerUnit:delete')")
    @CyOpeLogAnno(title = "system-banner与商家中间表管理-删除banner与商家中间表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "删除banner与商家中间表", notes = "删除banner与商家中间表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId, CmsBannerUnit cmsBannerUnit) {
        cmsBannerUnit.setBusinessId(String.valueOf(businessId));
        CyPersistModel data = cmsBannerUnitServiceImpl.remove(cmsBannerUnit);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:bannerUnit:update')")
    @CyOpeLogAnno(title = "system-banner与商家中间表管理-修改banner与商家中间表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value = "修改banner与商家中间表", notes = "修改banner与商家中间表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable int businessId, @Valid CmsBannerUnit cmsBannerUnit, BindingResult bindingResult) {
        cmsBannerUnit.setBusinessId(String.valueOf(businessId));
        CyPersistModel data = cmsBannerUnitServiceImpl.merge(cmsBannerUnit);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsBannerUnit);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:bannerUnit:query')")
    @CyOpeLogAnno(title = "system-banner与商家中间表管理-查询banner与商家中间表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询单一banner与商家中间表", notes = "查询单一banner与商家中间表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsBannerUnitServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:bannerUnit:list')")
    @CyOpeLogAnno(title = "system-banner与商家中间表管理-查询banner与商家中间表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询banner与商家中间表集合", notes = "查询banner与商家中间表集合")
    @GetMapping(value = "/listAll")
    public CyResult listAll(CmsBannerUnit cmsBannerUnit) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsBannerUnitServiceImpl.findAll(cmsBannerUnit));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:bannerUnit:list')")
    @CyOpeLogAnno(title = "system-banner与商家中间表管理-查询banner与商家中间表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "分页查询banner与商家中间表集合", notes = "分页查询banner与商家中间表集合")
    @GetMapping(value = "/list")
    public CyGridModel listByPagination(CmsBannerUnit cmsBannerUnit) {
        cmsBannerUnitServiceImpl.findAllByPagination(getPaginationUtility(), cmsBannerUnit);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-banner与商家中间表管理-查询banner与商家中间表", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出banner与商家中间表信息", notes = "导出banner与商家中间表信息")
    @GetMapping(value = "/export")
    public void outCmsBannerUnit(HttpServletResponse response, CmsBannerUnit cmsBannerUnit, @PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName = "";
        switch (excelId) {
            case "0":
                excelName = "banner与商家中间表信息.xls";
                break;
            case "1":
                excelName = "banner与商家中间表信息.xlsx";
                break;
            case "2":
                excelName = "banner与商家中间表信息.csv";
                break;
        }
        List<CmsBannerUnit> cmsBannerUnitList = cmsBannerUnitServiceImpl.export(cmsBannerUnit);
        CyEpExcelUtil.exportExcel(cmsBannerUnitList, "banner与商家中间表信息", "banner与商家中间表信息", CmsBannerUnit.class, excelName, response);
    }
}
