package org.rcisoft.business.cmsnews.controller;


/*固定导入*/

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.cmsnews.entity.CmsNews;
import org.rcisoft.business.cmsnews.service.CmsNewsService;
import org.rcisoft.business.cmsvideo.entity.CmsVideo;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyResultGenUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * Created by cy on 2024年6月3日 下午3:35:01.
 */
@RestController
@RequestMapping("/cmsnews")
public class CmsNewsController extends CyPaginationController<CmsNews> {


    @Autowired
    private CmsNewsService cmsNewsServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:news:add')")
    @CyOpeLogAnno(title = "system-文章表管理-新增文章表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value = "添加文章表", notes = "添加文章表")
    @PostMapping(value = "/add")
    public CyResult add(@Valid CmsNews cmsNews, BindingResult bindingResult) {
        CyPersistModel data = cmsNewsServiceImpl.persist(cmsNews);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsNews);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:news:delete')")
    @CyOpeLogAnno(title = "system-文章表管理-删除文章表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "逻辑删除文章表", notes = "逻辑删除文章表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}/{unitId:\\w+}")
    public CyResult deleteLogical(@PathVariable int businessId, @PathVariable String unitId, CmsNews cmsNews) {
        //字符串null
        if ("null".equals(unitId)) {
            unitId = null;
        }
        cmsNews.setBusinessId(businessId);
        cmsNews.setUnitId(unitId);
        CyPersistModel data = cmsNewsServiceImpl.removeLogical(cmsNews);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:news:delete')")
    @CyOpeLogAnno(title = "system-文章表管理-删除文章表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "删除文章表", notes = "删除文章表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId, CmsNews cmsNews) {
        cmsNews.setBusinessId(businessId);
        CyPersistModel data = cmsNewsServiceImpl.remove(cmsNews);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:news:update')")
    @CyOpeLogAnno(title = "system-文章表管理-修改文章表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value = "修改文章表", notes = "修改文章表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable int businessId, @Valid CmsNews cmsNews, BindingResult bindingResult) {
        cmsNews.setBusinessId(businessId);
        CyPersistModel data = cmsNewsServiceImpl.merge(cmsNews);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsNews);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:news:query')")
    @CyOpeLogAnno(title = "system-文章表管理-查询文章表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询单一文章表", notes = "查询单一文章表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsNewsServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:news:list')")
    @CyOpeLogAnno(title = "system-文章表管理-查询文章表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询文章表集合", notes = "查询文章表集合")
    @GetMapping(value = "/listAll")
    public CyResult listAll(CmsNews cmsNews) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsNewsServiceImpl.findAll(cmsNews));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:news:list')")
    @CyOpeLogAnno(title = "system-文章表管理-查询文章表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "分页查询文章表集合", notes = "分页查询文章表集合")
    @GetMapping(value = "/list")
    public CyGridModel listByPagination(CmsNews cmsNews) {
        cmsNewsServiceImpl.findAllByPagination(getPaginationUtility(), cmsNews);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-文章表管理-查询文章表", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出文章表信息", notes = "导出文章表信息")
    @GetMapping(value = "/export")
    public void outCmsNews(HttpServletResponse response, CmsNews cmsNews, @PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName = "";
        switch (excelId) {
            case "0":
                excelName = "文章表信息.xls";
                break;
            case "1":
                excelName = "文章表信息.xlsx";
                break;
            case "2":
                excelName = "文章表信息.csv";
                break;
        }
        List<CmsNews> cmsNewsList = cmsNewsServiceImpl.export(cmsNews);
        CyEpExcelUtil.exportExcel(cmsNewsList, "文章表信息", "文章表信息", CmsNews.class, excelName, response);
    }
    /*------------------------------通用---------------------------------------*/
    /*------------------------------管理端---------------------------------------*/



    /*------------------------------小程序---------------------------------------*/

    /**
     * 当前代言人绑定商家下的文章展示 分页查询置顶  需要传入商家id
     */
    @ApiOperation(value = "分页查询置顶文章", notes = "分页查询置顶文章")
    @GetMapping(value = "/miniProgram/listByTop")
    public CyResult listMiniProgram(CmsNews cmsNews) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsNewsServiceImpl.listMiniProgram(getPaginationUtility(), cmsNews));
    }

    /**
     * 当前代言人绑定商家下的视频展示 分页查询置顶  需要传入商家id
     */
    @ApiOperation(value = "分页查询置顶视频", notes = "分页查询置顶视频")
    @GetMapping(value = "/miniProgram/listByTopVideo")
    public CyResult listByTopVideo(CmsVideo cmsVideo) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsNewsServiceImpl.listByTopVideo(getPaginationUtility(), cmsVideo));
    }


    /**
     * 分类中查询当前用户负责的文章 和 视频
     */
    @ApiOperation(value = "分类中查询当前用户负责的文章 和 视频", notes = "分类中查询当前用户负责的文章 和 视频")
    @GetMapping(value = "/miniProgram/listByNewsAndVideo")
    public CyResult listByNewsAndVideo(CmsNews cmsNews) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsNewsServiceImpl.listByNewsAndVideo(getPaginationUtility(), cmsNews));
    }


    /**
     * 分类中查询当前用户负责的文章
     */
    @ApiOperation(value = "分类中查询当前用户负责的文章 和 视频", notes = "分类中查询当前用户负责的文章 和 视频")
    @GetMapping(value = "/miniProgram/listByClassification")
    public CyResult listByClassification(CmsNews cmsNews) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsNewsServiceImpl.listByClassification(getPaginationUtility(), cmsNews));
    }
}
