package org.rcisoft.business.cmsunitnewvideo.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.rcisoft.business.cmspriceclick.dao.CmsPriceClickRepository;
import org.rcisoft.business.cmspriceclick.entity.CmsPriceClick;
import org.rcisoft.business.cmsunitnewvideo.dao.CmsUnitNewVideoRepository;
import org.rcisoft.business.cmsunitnewvideo.entity.CmsUnitNewVideo;
import org.rcisoft.business.cmsunitnewvideo.service.CmsUnitNewVideoService;
import org.rcisoft.business.sysunit.dao.SysUnitRepository;
import org.rcisoft.business.sysunit.entity.SysUnit;
import org.rcisoft.business.sysuser.entity.AddSysUser;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.sys.wbac.user.dao.SysUserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Created by cy on 2024年6月6日 上午10:17:38.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class CmsUnitNewVideoServiceImpl extends ServiceImpl<CmsUnitNewVideoRepository, CmsUnitNewVideo> implements CmsUnitNewVideoService {

    @Autowired
    private CmsPriceClickRepository cmsPriceClickRepository;
    @Autowired
    private SysUnitRepository sysUnitRepository;
    @Autowired
    private SysUserRepository sysUserRepository;
    @Autowired
    private CmsUnitNewVideoRepository cmsUnitNewVideoRepository;

    /**
     * 保存 文章视频分配中间表
     *
     * @param cmsUnitNewVideo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel addOrUpdate(CmsUnitNewVideo cmsUnitNewVideo) {
        //查询数据 是否包含
        LambdaQueryWrapper<CmsUnitNewVideo> qw = new LambdaQueryWrapper<>();
        qw.eq(CmsUnitNewVideo::getNewsVideoId, cmsUnitNewVideo.getNewsVideoId())
                .eq(CmsUnitNewVideo::getType, cmsUnitNewVideo.getType());
        //目前数据库存在数据
        List<CmsUnitNewVideo> cmsUnitNewVideos = baseMapper.selectList(qw);
        //第一个新添加数据  传入有查出没有
        List<String> incomingUnitIds = Arrays.stream(cmsUnitNewVideo.getUnitId().split(",")).map(String::trim).collect(Collectors.toList());
        //将查出来的数据转成集合
        List<String> figureOutUnitIds = cmsUnitNewVideos.stream().map(CmsUnitNewVideo::getUnitId).collect(Collectors.toList());
        //将新添加的数据和查出来的数据做对比 保留只有传入有的 然后创建新的集合
        List<String> retainAll = incomingUnitIds.stream().filter(item -> !figureOutUnitIds.contains(item)).collect(Collectors.toList());
        //对商家做添加
        retainAll.forEach(unitId -> {
            //1 创建关联对象
            CmsUnitNewVideo cmsUnitNewVideo1 = new CmsUnitNewVideo();
            cmsUnitNewVideo1.setNewsVideoId(cmsUnitNewVideo.getNewsVideoId());
            cmsUnitNewVideo1.setType(cmsUnitNewVideo.getType());
            cmsUnitNewVideo1.setUnitId(unitId);
            cmsUnitNewVideoRepository.insert(cmsUnitNewVideo1);
          /*  //查询当前商家下的代言人
            List<Integer> integers = sysUnitRepository.selectByUnitId(unitId);
            ArrayList<CmsPriceClick> cmsPriceClicks = new ArrayList<>();
            //代言人循环创建
            for (Integer integer : integers) {
                CmsPriceClick cmsPriceClick = new CmsPriceClick();
                cmsPriceClick.setType(cmsUnitNewVideo.getType());
                cmsPriceClick.setNewsVideoId(cmsUnitNewVideo.getNewsVideoId());
                cmsPriceClick.setUserId(integer);
                cmsPriceClick.setUnitId(Integer.valueOf(unitId));
                cmsPriceClick.setPrice(BigDecimal.valueOf(0));
                cmsPriceClick.setDelFlag(String.valueOf(0));
                cmsPriceClick.setFlag(String.valueOf(1));
                cmsPriceClick.setClickThroughRate(0);
                cmsPriceClick.setCreateBy(CyUserUtil.getAuthenBusinessId());
                cmsPriceClick.setUpdateBy(CyUserUtil.getAuthenBusinessId());
                cmsPriceClick.setCreateDate(new Date());
                cmsPriceClick.setUpdateDate(new Date());
                cmsPriceClicks.add(cmsPriceClick);
            }*/
//            if (cmsPriceClicks.size() > 0) {
//                //添加
//                cmsPriceClickRepository.insertList(cmsPriceClicks);
//            }
        });
        //将新添加的数据和查出来的数据做对比 保留只有查出有的 然后创建新的集合
        List<String> retainAll1 = figureOutUnitIds.stream().filter(item -> !incomingUnitIds.contains(item)).collect(Collectors.toList());
        //对商家做删除
        retainAll1.forEach(unitId -> {
            //删除新闻或商家代言表
            cmsUnitNewVideoRepository.deleteByUnitIdAndNewsVideoId(unitId, cmsUnitNewVideo.getNewsVideoId(), cmsUnitNewVideo.getType());
            //对当前新闻和当前商家下的推广率 修改
            cmsPriceClickRepository.updateTheUserByNewVideoIdUnitId(unitId, cmsUnitNewVideo.getNewsVideoId(), cmsUnitNewVideo.getType());
        });
        return new CyPersistModel(1);
    }

    @Override
    public void deleteTheUserByMerchantId(SysUnit sysUnit) {
        baseMapper.deleteTheUserByMerchantId(sysUnit);
    }


    private Integer addData(CmsUnitNewVideo cmsUnitNewVideo) {
        int line = 0;
        if (cmsUnitNewVideo.getUnitId() != null && cmsUnitNewVideo.getUnitId().contains(",")) {
            String[] unitIds = cmsUnitNewVideo.getUnitId().split(",");
            for (String unitId : unitIds) {
                cmsUnitNewVideo.setUnitId(unitId);
                line = baseMapper.insert(cmsUnitNewVideo);
            }
        } else {
            line = baseMapper.insert(cmsUnitNewVideo);
        }
        return line;
    }

    public void queryVideoOrArticleRelationships(AddSysUser sysUser, ArrayList<String> collect) {
        collect.forEach(deptId -> {
            //如果当前商家拥有视频或者文章 需要新增 代言人绑定视频或者文章
            List<CmsUnitNewVideo> cmsUnitNewVideos = cmsUnitNewVideoRepository.queryVideoOrArticleRelationships(Integer.valueOf(deptId));
            if (CollectionUtils.isNotEmpty(cmsUnitNewVideos)) {
                for (CmsUnitNewVideo cmsUnitNewVideo : cmsUnitNewVideos) {
                    CmsPriceClick cmsPriceClick = new CmsPriceClick();
                    cmsPriceClick.setNewsVideoId(cmsUnitNewVideo.getNewsVideoId());
                    cmsPriceClick.setUnitId(Integer.valueOf(cmsUnitNewVideo.getUnitId()));
                    cmsPriceClick.setUserId(sysUser.getBusinessId());
                    cmsPriceClick.setType(cmsUnitNewVideo.getType());
                    cmsPriceClick.setFlag(String.valueOf(1));
                    cmsPriceClick.setClickThroughRate(0);
                    cmsPriceClick.setPrice(BigDecimal.valueOf(0));
                    cmsPriceClickRepository.insert(cmsPriceClick);
                }
            }
        });
    }

    @Override
    public List<CmsUnitNewVideo> listAllByUnitIdType(CmsUnitNewVideo cmsUnitNewVideo) {
        return  cmsUnitNewVideoRepository.listAllByUnitIdType(cmsUnitNewVideo);
    }
}
