package org.rcisoft.business.cmsvideo.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.cmspriceclick.dao.CmsPriceClickRepository;
import org.rcisoft.business.cmsunitnewvideo.dao.CmsUnitNewVideoRepository;
import org.rcisoft.business.cmsunitnewvideo.entity.CmsUnitNewVideo;
import org.rcisoft.business.cmsvideo.dao.CmsVideoRepository;
import org.rcisoft.business.cmsvideo.entity.CmsVideo;
import org.rcisoft.business.cmsvideo.service.CmsVideoService;
import org.rcisoft.business.ossinfo.dao.OssInfoRepository;
import org.rcisoft.business.ossinfo.entity.OssInfo;
import org.rcisoft.business.sysunit.dao.SysUnitRepository;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.component.CyInitComp;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.util.RSAUtil;
import org.rcisoft.result.ResultServiceEnums;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * Created by cy on 2024年6月3日 下午3:35:01.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class CmsVideoServiceImpl extends ServiceImpl<CmsVideoRepository, CmsVideo> implements CmsVideoService {

    @Autowired
    private CmsUnitNewVideoRepository cmsUnitNewVideoRepository;

    @Autowired
    private Global global;

    @Autowired
    private OssInfoRepository ossInfoRepository;

    @Autowired
    private CyInitComp cyInitComp;
    @Autowired
    private SysUnitRepository sysUnitRepository;
    @Autowired
    private CmsPriceClickRepository cmsPriceClickRepository;

    /**
     * 保存 视频表
     *
     * @param cmsVideo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(CmsVideo cmsVideo) {
        //增加操作
        int line = baseMapper.insert(cmsVideo);
        //判断如果当前用户为商家管理员 将这个与商家管理员进行绑定
        if (CyUserUtil.getAuthenRoleIds().equals(cyInitComp.unit[0])) {
            //创建绑定
            CmsUnitNewVideo cmsUnitNewVideo = new CmsUnitNewVideo();
            cmsUnitNewVideo.setNewsVideoId(cmsVideo.getBusinessId());
            cmsUnitNewVideo.setUnitId(cmsVideo.getUnitId());
            cmsUnitNewVideo.setType("1");
            cmsUnitNewVideoRepository.insert(cmsUnitNewVideo);
            //查询当前商家下的代言人
//            List<Integer> integers = sysUnitRepository.selectByUnitId(cmsVideo.getUnitId());
            //创建文章视频推广
//            for (Integer integer : integers) {
//                CmsPriceClick cmsPriceClick = new CmsPriceClick();
//                cmsPriceClick.setType(cmsUnitNewVideo.getType());
//                cmsPriceClick.setNewsVideoId(cmsUnitNewVideo.getNewsVideoId());
//                cmsPriceClick.setUserId(integer);
//                cmsPriceClick.setUnitId(Integer.valueOf(cmsVideo.getUnitId()));
//                cmsPriceClick.setPrice(BigDecimal.valueOf(0));
//                cmsPriceClick.setDelFlag(String.valueOf(0));
//                cmsPriceClick.setFlag(String.valueOf(1));
//                cmsPriceClick.setClickThroughRate(0);
//                cmsPriceClick.setCreateBy(CyUserUtil.getAuthenBusinessId());
//                cmsPriceClick.setUpdateBy(CyUserUtil.getAuthenBusinessId());
//                cmsPriceClick.setCreateDate(new Date());
//                cmsPriceClick.setUpdateDate(new Date());
//                cmsPriceClickRepository.insert(cmsPriceClick);
//            }


        }
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                cmsVideo.getBusinessId() + "的视频表信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 视频表
     *
     * @param cmsVideo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(CmsVideo cmsVideo) {
        int line = baseMapper.realDelete(cmsVideo);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                cmsVideo.getBusinessId() + "的视频表信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 视频表
     *
     * @param cmsVideo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(CmsVideo cmsVideo) {
        //判断当前用户是否为超级管理员
        if (CyUserUtil.getAuthenRoleIds().equals(cyInitComp.admin[0])) {
            cmsVideo.setDeleted();
            //逻辑删除本表
            int line = baseMapper.deleteById(cmsVideo);
            //关联表
            cmsUnitNewVideoRepository.deleteByNewsVideoId(cmsVideo.getBusinessId(), "1");
            //推广表逻辑删除
            cmsPriceClickRepository.updateTheUserByNewVideoId(cmsVideo.getBusinessId(), "1");
        }
        //判断当前用户是否为商家管理员
        if (CyUserUtil.getAuthenRoleIds().equals(cyInitComp.unit[0])) {
            //关联表
            cmsUnitNewVideoRepository.deleteByUnitIdAndNewsVideoId(cmsVideo.getUnitId(), cmsVideo.getBusinessId(), "1");
            //推广表逻辑删除
            cmsPriceClickRepository.updateTheUserByNewVideoIdUnitId(cmsVideo.getUnitId(), cmsVideo.getBusinessId(), "1");
        }
        return new CyPersistModel(1);
    }

    /**
     * 修改 视频表
     *
     * @param cmsVideo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(CmsVideo cmsVideo) {
        int line = baseMapper.updateById(cmsVideo);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + cmsVideo.getBusinessId() + "的视频表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 视频表
     *
     * @param id
     * @return
     */
    @Override
    public CmsVideo findById(int id) {
        CmsVideo cmsVideo = baseMapper.selectById(id);
        //图片url
        if (cmsVideo != null && StringUtils.isNotEmpty(cmsVideo.getDefaultUrl())) {
            OssInfo ossInfo = ossInfoRepository.selectById(cmsVideo.getDefaultUrl());
            if (ossInfo.getPath() != null) {
                cmsVideo.setPath(global.getBase_Discovery() + ossInfo.getPath());
//                cmsVideo.setVideoName(ossInfo.getOriginalName());
            }

        }
        //视频url
        if (cmsVideo != null && StringUtils.isNotEmpty(cmsVideo.getVideoUrlId())) {
            OssInfo ossInfo = ossInfoRepository.selectById(cmsVideo.getVideoUrlId());
            if (ossInfo.getPath() != null) {
//                cmsVideo.setPath(global.getBase_Discovery() + ossInfo.getPath());
                cmsVideo.setVideoName(ossInfo.getOriginalName());
            }
        }

        return cmsVideo;
    }

    /**
     * 分页查询 视频表
     *
     * @param cmsVideo
     * @return
     */
    @Override
    public IPage<CmsVideo> findAllByPagination(CyPageInfo<CmsVideo> paginationUtility,
                                               CmsVideo cmsVideo) {
        //获取当前人身份
        //超级管理员

        if (CyUserUtil.getAuthenRoleIds().equals(cyInitComp.admin[0])) {
            IPage<CmsVideo> page = baseMapper.superAdminPaginatedQuery(paginationUtility, cmsVideo);
            return page;
        }
        //商家管理员
        if (CyUserUtil.getAuthenRoleIds().equals(cyInitComp.unit[0])) {
            IPage<CmsVideo> page = baseMapper.businessAdminsPaginatedQuery(paginationUtility, cmsVideo);
            return page;
        }

//
//        IPage<CmsVideo> page = baseMapper.queryCmsVideosPaged(paginationUtility, cmsVideo);
//        List<CmsVideo> records = page.getRecords();
//        //查询文章所属商家信息
//        LambdaQueryWrapper<CmsUnitNewVideo> qw = new LambdaQueryWrapper<>();
//        qw.eq(CmsUnitNewVideo::getType, 1);
//        List<CmsUnitNewVideo> cmsUnitNewVideos = cmsUnitNewVideoRepository.selectList(qw);
//        //根据文章id查询商家信息
//        Map<Integer, List<CmsUnitNewVideo>> map = cmsUnitNewVideos.stream().collect(Collectors.groupingBy(CmsUnitNewVideo::getNewsVideoId));
//        for (CmsVideo cmsVideo1 : records) {
//            if (cmsVideo1 != null && map.containsKey(cmsVideo1.getBusinessId()) && map.get(cmsVideo1.getBusinessId()) != null) {
//                List<CmsUnitNewVideo> cmsUnitNewVideos1 = map.get(cmsVideo1.getBusinessId());
//                List<String> deptIds = cmsUnitNewVideos1.stream().map(CmsUnitNewVideo::getUnitId).collect(Collectors.toList());
//                cmsVideo1.setUnitId(deptIds.toString());
//            }
//            if (cmsVideo1.getPath() != null) {
//                cmsVideo1.setPath(global.getBase_Discovery() + cmsVideo1.getPath());
//            }
//        }
//        return page;
        return null;
    }

    /**
     * 查询list 视频表
     *
     * @param cmsVideo
     * @return
     */
    @Override
    public List<CmsVideo> findAll(CmsVideo cmsVideo) {
        return baseMapper.queryCmsVideos(cmsVideo);

    }

    /**
     * 导出视频表
     *
     * @return
     */
    @Override
    public List<CmsVideo> export(CmsVideo cmsVideo) {
        List<CmsVideo> cmsVideoList = baseMapper.queryCmsVideos(cmsVideo);
        return cmsVideoList;
    }


    @Override
    public CmsVideo getVideoUrl(CmsVideo cmsVideo) {
        //使用id 查询 路径
        OssInfo ossInfo = ossInfoRepository.selectById(cmsVideo.getVideoUrlId());
        try {
            if (ossInfo != null && StringUtils.isNotEmpty(ossInfo.getPath())) {
                String path = RSAUtil.encryptByPublicKey(null, global.getBase_Discovery() + ossInfo.getPath());
                cmsVideo.setPath(path);
            }
        } catch (Exception e) {
            throw new CyServiceException(ResultServiceEnums.URL_ENCODE.getCode(), ResultServiceEnums.URL_ENCODE.getMessage());
        }
        return cmsVideo;
    }

    @Override
    public IPage<CmsVideo> listByClassification(CyPageInfo<CmsVideo> paginationUtility, CmsVideo cmsVideo) {
        //超级管理员
        if (CyUserUtil.getAuthenRoleIds().equals(cyInitComp.admin[0])) {
            IPage<CmsVideo> page = baseMapper.listByClassificationAdmin(paginationUtility, cmsVideo);
            List<CmsVideo> records = page.getRecords();
            for (CmsVideo cmsVideo1 : records) {
                if (cmsVideo1.getPath() != null) {
                    cmsVideo1.setPath(global.getBase_Discovery() + cmsVideo1.getPath());
                }
            }
            return page;
        }
        //商家管理员
        if (CyUserUtil.getAuthenRoleIds().equals(cyInitComp.unit[0])) {
            cmsVideo.setUserId(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
            IPage<CmsVideo> page = baseMapper.listByClassificationMerchants(paginationUtility, cmsVideo);
            List<CmsVideo> records = page.getRecords();
            for (CmsVideo cmsVideo1 : records) {
                if (cmsVideo1.getPath() != null) {
                    cmsVideo1.setPath(global.getBase_Discovery() + cmsVideo1.getPath());
                }
            }
            return page;
        }


        //校验是否包含商家信息
        cmsVideo.setUserId(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
        IPage<CmsVideo> page = baseMapper.listByClassification(paginationUtility, cmsVideo);
        List<CmsVideo> records = page.getRecords();
        for (CmsVideo cmsVideo1 : records) {
            if (cmsVideo1.getPath() != null) {
                cmsVideo1.setPath(global.getBase_Discovery() + cmsVideo1.getPath());
            }
        }
        return page;
    }

}
