package org.rcisoft.business.crawler.controller;

import lombok.SneakyThrows;
import org.rcisoft.business.cmstask.entity.CmsTask;
import org.rcisoft.business.cmstask.service.CmsTaskService;
import org.rcisoft.business.crawler.api.WeiXinApi;
import org.rcisoft.business.crawler.model.WxResultBody;
import org.rcisoft.business.crawler.service.PublicAccountCrawlerService;
import org.rcisoft.business.crawler.service.WebsiteCrawlerService;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.InputStream;
import java.util.Date;

/**
 * 爬虫controller
 *
 * @author gxk
 */
@RestController
@RequestMapping("/crawler")
public class CrawlerController {

    @Autowired
    private WebsiteCrawlerService crawlerService;

    @Autowired
    private PublicAccountCrawlerService publicAccountCrawlerService;
    @Autowired
    private CmsTaskService cmsTaskService;

    /**
     * web端爬取页面数据 + 定时爬取
     */
    @GetMapping(value = "/start")
    public CyResult start() throws Exception {
        CmsTask cmsTask = cmsTaskService.add("web" + new Date());
        //将用户拿出来
        String authenBusinessId = CyUserUtil.getAuthenBusinessId();

        new Thread(() -> {
            try {
                //爬取接口
                Integer count = crawlerService.startCrawler();
                //爬取完成
                cmsTask.setStatus("1");
                cmsTask.setUpdateBy(authenBusinessId);
                cmsTask.setNum(count);
                cmsTaskService.merge(cmsTask);
            } catch (Exception e) {
                System.out.println("------------------" + e);
                //爬取失败
                cmsTask.setStatus("2");
                cmsTask.setUpdateBy(authenBusinessId);
                cmsTaskService.merge(cmsTask);
            }
        }).start();
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsTask.getBusinessId());
    }

    /**
     * 获取公众号登陆二维码
     *
     * @return
     */
    @GetMapping("/image")
    @SneakyThrows
    public ResponseEntity<byte[]> getImage() {
        // POST请求开始登录接口，初始化cookie
        String sessionId = String.valueOf(System.currentTimeMillis()) + (int) (Math.random() * 100);
        WxResultBody wxResultBody = WeiXinApi.startLogin(sessionId);
        System.out.println("---请求开始登录接口 返回结果:" + wxResultBody.toString());

        // 请求获取二维码图片接口，得到流
        InputStream qrCodeIs = WeiXinApi.getQRCode();

        // 将输入流（InputStream）中的所有字节读取到一个字节数组
        byte[] imageBytes = qrCodeIs.readAllBytes();

        // 设置响应头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.parseMediaType("image/png"));
        headers.add(HttpHeaders.CONTENT_DISPOSITION, "inline; filename=\"" + "code" + "\"");

        return new ResponseEntity<>(imageBytes, headers, HttpStatus.OK);
    }

    /**
     * 根据公众号名查询其文章
     */
    @GetMapping("/scan/{publicAccountNames}")
    public CyResult scanImage(@PathVariable String publicAccountNames) throws Exception {
        CmsTask cmsTask = cmsTaskService.add(publicAccountNames + new Date());
        //将用户拿出来
        String authenBusinessId = CyUserUtil.getAuthenBusinessId();
//        new Thread(() -> {
//        调用接口
        try {
            Integer count = publicAccountCrawlerService.scanImage(publicAccountNames);
            cmsTask.setStatus("1");
            cmsTask.setUpdateBy(authenBusinessId);
            cmsTask.setNum(count);
            cmsTaskService.merge(cmsTask);
        } catch (Exception e) {
            //抛出异常
            e.printStackTrace();

            System.out.println("------------------" + e);
            //出现异常 修改 数据库 任务表数据 并 返回
            cmsTask.setStatus("2");
            cmsTask.setUpdateBy(authenBusinessId);
            cmsTaskService.merge(cmsTask);
        }
//        }).start();
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                cmsTask.getBusinessId());
    }

}
