package org.rcisoft.business.crawler.scheduled;

import org.rcisoft.business.cmstask.entity.CmsTask;
import org.rcisoft.business.cmstask.service.CmsTaskService;
import org.rcisoft.business.crawler.service.WebsiteCrawlerService;
import org.rcisoft.sys.dictionary.dto.DictDataDTO;
import org.rcisoft.sys.dictionary.entity.DictData;
import org.rcisoft.sys.dictionary.service.DictDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;


/**
 * 爬虫 定时任务
 *
 * @author gxk
 */
@Component
public class ScheduledTasks {

    @Autowired
    private WebsiteCrawlerService crawlerService;

    @Autowired
    private DictDataService dictDataService;

    @Value("${cron.expression}")
    private String cronExpression;
    @Autowired
    private CmsTaskService cmsTaskService;

    /**
     * 5分钟爬取一次
     */
    @Scheduled(cron = "#{@scheduledTasks.getCronExpression()}")
    public void crawlerTask() {
        CmsTask cmsTask = cmsTaskService.add("定时任务" + new Date());
        try {
            // 根据字典判断是否开启定时爬虫
            DictDataDTO data = new DictDataDTO();
            data.setDictType("timing_crawler");
            List<DictData> dataList = dictDataService.findAll(data);
            if (dataList.size() > 0 && "1".equals(dataList.get(0).getFlag())) {
                System.out.println("Task performed on: " + new java.util.Date());
                crawlerService.startCrawler();
            }
            cmsTask.setStatus("1");
            cmsTaskService.merge(cmsTask);
        } catch (Exception e) {
            //爬取失败
            cmsTask.setStatus("2");
            cmsTaskService.merge(cmsTask);
        }

    }

    /**
     * 提供一个方法返回cron表达式，这样可以被SpEL引用
     *
     * @return
     */
    public String getCronExpression() {
        return cronExpression;
    }
}
