package org.rcisoft.business.sysunit.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.cmspriceclick.entity.CmsPriceClick;
import org.rcisoft.business.sysunit.dao.SysUnitRepository;
import org.rcisoft.business.sysunit.dto.StatisticsDto;
import org.rcisoft.business.sysunit.entity.SysUnit;
import org.rcisoft.business.sysunit.service.SysUnitService;
import org.rcisoft.business.sysuser.entity.SysUser;
import org.rcisoft.business.sysuser.service.impl.SysUserServiceImpl;
import org.rcisoft.core.component.CyInitComp;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.result.ResultServiceEnums;
import org.rcisoft.sys.rbac.user.entity.SysUserRbac;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Created by cy on 2024年6月3日 下午3:35:01.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class SysUnitServiceImpl extends ServiceImpl<SysUnitRepository, SysUnit> implements SysUnitService {

    @Autowired
    private SysUserServiceImpl sysUserService;
    @Autowired
    private CyInitComp cyInitComp;

    /**
     * 保存 单位管理
     *
     * @param sysUnit
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(SysUnit sysUnit) {
        //增加操作
        int line = baseMapper.insert(sysUnit);
        //添加unit_user表
        baseMapper.insertByUnitUser(sysUnit.getBusinessId(), sysUnit.getUserId());
        //添加用户角色
        SysUser user = sysUserService.findById(sysUnit.getUserId());
        if (ObjectUtil.isNotEmpty(user)) {
            user.setDeptId(BigInteger.valueOf(sysUnit.getBusinessId()));
            sysUserService.updateById(user);
        }
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                sysUnit.getBusinessId() + "的单位管理信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 单位管理
     *
     * @param sysUnit
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(SysUnit sysUnit) {
        int line = baseMapper.realDelete(sysUnit);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                sysUnit.getBusinessId() + "的单位管理信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 单位管理
     *
     * @param sysUnit
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(SysUnit sysUnit) {
        sysUnit.setDeleted();
        int line = baseMapper.deleteById(sysUnit);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                sysUnit.getBusinessId() + "的单位管理信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 单位管理
     *
     * @param sysUnit
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(SysUnit sysUnit) {
        int line = baseMapper.updateById(sysUnit);
        //根据用户id查询用户信息并且修改
        if (sysUnit.getUserId() != null) {
            SysUser user = sysUserService.findById(sysUnit.getUserId());
            user.setDeptId(BigInteger.valueOf(sysUnit.getBusinessId()));
            sysUserService.updateById(user);
        }
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + sysUnit.getBusinessId() + "的单位管理信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 单位管理
     *
     * @param id
     * @return
     */
    @Override
    public SysUnit findById(int id) {
        return baseMapper.selectById(id);
    }

    /**
     * 分页查询 单位管理
     *
     * @param sysUnit
     * @return
     */
    @Override
    public IPage<SysUnit> findAllByPagination(CyPageInfo<SysUnit> paginationUtility,
                                              SysUnit sysUnit) {
        return baseMapper.querySysUnitsPaged(paginationUtility, sysUnit);

    }

    /**
     * 查询list 单位管理
     *
     * @param sysUnit
     * @return
     */
    @Override
    public List<SysUnit> findAll(SysUnit sysUnit) {
        //判断当前用户是超级管理员还是商家管理员
        //判断当前用户角色 来决定数据范围 1 超级管理员 2 商家管理员
        String authenRoleIds = CyUserUtil.getAuthenRoleIds();
        //authenRoleIds的场景只会存在一个数据 判断
        // 1 = 超级管理员
        if (cyInitComp.admin[0].equals(authenRoleIds)) {
            //分页配置
            return baseMapper.querySysUnits(sysUnit);
        }
        // 2 = 商家管理员
        if (cyInitComp.unit[0].equals(authenRoleIds)) {
            //当前用户拥有商家
            //用户id
            sysUnit.setUserId(Integer.valueOf(CyUserUtil.getAuthenBusinessId()));
            sysUnit.setFlag("1");
            return baseMapper.querySysUnits(sysUnit);
        }
        return null;


    }

    /**
     * 导出单位管理
     *
     * @return
     */
    @Override
    public List<SysUnit> export(SysUnit sysUnit) {
        List<SysUnit> sysUnitList = baseMapper.querySysUnits(sysUnit);
        return sysUnitList;
    }

    @Override
    public List<SysUserRbac> listAllUser(SysUserRbac sysUserRbac) {
        return baseMapper.listAllUser(sysUserRbac);
    }

    /**
     * 查询商家管理员信息
     *
     * @param sysUserRbac
     * @return
     */
    @Override
    public List<SysUserRbac> queryUnitAdmin(SysUserRbac sysUserRbac) {
        return baseMapper.queryUnitAdmin(sysUserRbac);
    }

    @Override
    public SysUnit statistics(SysUnit sysUnit) {
        //通过商家id查询出当前商家的数据统计
        return baseMapper.statistics(sysUnit);
    }

    @Override
    public void deleteTheUserByMerchantId(SysUnit sysUnit) {
        baseMapper.deleteTheUserByMerchantId(sysUnit);
    }

    /**
     * 饼状图统计
     *
     * @param sysUnit
     * @return
     */
    @Override
    public Map<Integer, List<StatisticsDto>> statisticsByTypePieChart(SysUnit sysUnit) {
        List<StatisticsDto> dtos = baseMapper.statisticsByTypePieChart(sysUnit);
        //根据类型分组
        Map<Integer, List<StatisticsDto>> map = dtos.stream().collect(Collectors.groupingBy(StatisticsDto::getType));
        return map;
    }

    @Override
    public List<StatisticsDto> statisticsByType(SysUnit dto) {
        //文章
        if ("0".equals(dto.getType())) {
            List<StatisticsDto> dtos = baseMapper.selectByNewsRate(dto);
            return dtos;
        }
        //视频
        if ("1".equals(dto.getType())) {
            List<StatisticsDto> dtos = baseMapper.selectByVideoRate(dto);
            return dtos;
        }
        return null;
    }

    @Override
    public List<CmsPriceClick> aListOfPromotionAmounts(SysUnit sysUnit) {
        if (sysUnit.getBusinessId() != null) {
            return baseMapper.aListOfPromotionAmounts(sysUnit);
        }
        return null;
    }

    /**
     * 查询当前登录人的商家信息
     *
     * @return
     */
    @Override
    public List<SysUnit> findUnitByUnit() {
        String userId = CyUserUtil.getAuthenBusinessId();
        if (StringUtils.isEmpty(userId)) {
            throw new CyServiceException(ResultServiceEnums.NOT_LOGIN.getCode(),
                    ResultServiceEnums.NOT_LOGIN.getMessage());
        }
        //超级管理员
        if (cyInitComp.admin[0].equals(CyUserUtil.getAuthenRoleIds())) {
            List<SysUnit> allUnit = baseMapper.findAllUnit();
            SysUnit sysUnit = new SysUnit();
            sysUnit.setUnitName("全部");
            allUnit.add(sysUnit);
            return allUnit;
        }
        //商家
        if (cyInitComp.unit[0].equals(CyUserUtil.getAuthenRoleIds())) {
            return baseMapper.findUnit(userId);
        }
        //根据用户id查询当前人的商家信息 （代言人）
        return baseMapper.findUnitByUnit(userId);
    }

    @Override
    public List<StatisticsDto> statisticsByTypeBarChart(SysUnit sysUnit) {
        //查询当前代言人的文章/视频 总推广次数
        return baseMapper.statisticsByTypeBarChart(sysUnit);
    }

    /**
     * 获取商家名称
     *
     * @return
     */
    public String[] selectByUintName() {
        return baseMapper.selectByUintName();
    }

    public ArrayList<Integer> findUserId(Integer unitId) {
        return baseMapper.findUserId(unitId);
    }
}
