package org.rcisoft.business.sysunitnews.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.exception.CyServiceException;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;


import org.rcisoft.business.sysunitnews.entity.SysUnitNews;
import org.rcisoft.business.sysunitnews.service.SysUnitNewsService;

import java.util.List;

/**
* Created by cy on 2024年6月3日 下午3:35:01.
*/
@RestController
@RequestMapping("/sysunitnews")
public class SysUnitNewsController extends CyPaginationController<SysUnitNews> {


    @Autowired
    private SysUnitNewsService sysUnitNewsServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:unitNews:add')")
    @CyOpeLogAnno(title = "system-文章和单位中间表管理-新增文章和单位中间表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value="添加文章和单位中间表", notes="添加文章和单位中间表")
    @PostMapping(value = "/add")
    public CyResult add(@Valid SysUnitNews sysUnitNews, BindingResult bindingResult) {
        CyPersistModel data = sysUnitNewsServiceImpl.persist(sysUnitNews);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        sysUnitNews);
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:unitNews:delete')")
    @CyOpeLogAnno(title = "system-文章和单位中间表管理-删除文章和单位中间表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="删除文章和单位中间表", notes="删除文章和单位中间表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId,SysUnitNews sysUnitNews) {
        sysUnitNews.setBusinessId(businessId);
        CyPersistModel data = sysUnitNewsServiceImpl.remove(sysUnitNews);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:unitNews:update')")
    @CyOpeLogAnno(title = "system-文章和单位中间表管理-修改文章和单位中间表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value="修改文章和单位中间表", notes="修改文章和单位中间表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable int businessId, @Valid SysUnitNews sysUnitNews, BindingResult bindingResult) {
        sysUnitNews.setBusinessId(businessId);
        CyPersistModel data = sysUnitNewsServiceImpl.merge(sysUnitNews);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        sysUnitNews);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:unitNews:query')")
    @CyOpeLogAnno(title = "system-文章和单位中间表管理-查询文章和单位中间表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询单一文章和单位中间表", notes="查询单一文章和单位中间表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        sysUnitNewsServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:unitNews:list')")
    @CyOpeLogAnno(title = "system-文章和单位中间表管理-查询文章和单位中间表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询文章和单位中间表集合", notes="查询文章和单位中间表集合")
    @GetMapping(value = "/listAll")
    public CyResult listAll(SysUnitNews sysUnitNews) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            sysUnitNewsServiceImpl.findAll(sysUnitNews));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:unitNews:list')")
    @CyOpeLogAnno(title = "system-文章和单位中间表管理-查询文章和单位中间表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="分页查询文章和单位中间表集合", notes="分页查询文章和单位中间表集合")
    @GetMapping(value = "/list")
    public CyGridModel listByPagination(SysUnitNews sysUnitNews) {
        sysUnitNewsServiceImpl.findAllByPagination(getPaginationUtility(), sysUnitNews);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-文章和单位中间表管理-查询文章和单位中间表", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出文章和单位中间表信息", notes = "导出文章和单位中间表信息")
    @GetMapping(value = "/export")
    public void outSysUnitNews(HttpServletResponse response,SysUnitNews sysUnitNews,@PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName="";
        switch(excelId){
            case "0": excelName="文章和单位中间表信息.xls";break;
            case "1": excelName="文章和单位中间表信息.xlsx";break;
            case "2": excelName="文章和单位中间表信息.csv";break;
        }
        List<SysUnitNews> sysUnitNewsList = sysUnitNewsServiceImpl.export(sysUnitNews);
        CyEpExcelUtil.exportExcel(sysUnitNewsList, "文章和单位中间表信息", "文章和单位中间表信息", SysUnitNews.class, excelName, response);
    }
}
