package org.rcisoft.business.sysuser.controller;


/*固定导入*/

import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import com.alipay.api.internal.util.file.IOUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.rcisoft.business.cmsunitnewvideo.service.CmsUnitNewVideoService;
import org.rcisoft.business.sysuser.entity.AddSysUser;
import org.rcisoft.business.sysuser.entity.SysUser;
import org.rcisoft.business.sysuser.service.AdminSysUserService;
import org.rcisoft.common.component.Global;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.component.CyInitComp;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyResultGenUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by cy on 2024年6月4日 上午10:19:34.
 */
@Slf4j
@RestController
@RequestMapping("/sysuser")
public class SysUserController extends CyPaginationController<SysUser> {


    private static final String STUDENT_IMPORT_TEMPLATE_NAME = "Spokesperson_template.xlsx";
    @Autowired
    private AdminSysUserService sysUserServiceImpl;
    @Autowired
    private Global global;
    @Autowired
    private CyInitComp cyInitComp;
    @Autowired
    private CmsUnitNewVideoService cmsUnitNewVideoService;

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:add')")
    @CyOpeLogAnno(title = "system-用户表管理-新增代言人信息表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value = "添加代言人信息", notes = "添加代言人信息")
    @PostMapping(value = "/add")
    public CyResult add(@Valid AddSysUser sysUser, BindingResult bindingResult) {
        ArrayList<Integer> integers = new ArrayList<>();
        integers.add(Integer.valueOf(cyInitComp.individual[0]));
        sysUser.setRoleList(integers);
        sysUser.setDeptId(null);
        CyPersistModel data = sysUserServiceImpl.persist(sysUser);
        //当前用户已经创建好，但是之前添加文章和视频没有添加
//        ArrayList<String> collect = Arrays.stream(sysUser.getDeptIdPrepares().split(",")).collect(Collectors.toCollection(ArrayList::new));
        //给用户添加历史文章和视频
//        cmsUnitNewVideoService.queryVideoOrArticleRelationships(sysUser, collect);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUser);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:delete')")
    @CyOpeLogAnno(title = "system-用户表管理-删除用户表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "逻辑删除用户表", notes = "逻辑删除用户表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}/{unitId:\\w+}")
    public CyResult deleteLogical(@PathVariable int businessId, @PathVariable String unitId, SysUser sysUser) {
        sysUser.setBusinessId(businessId);
        if (!"null".equals(unitId)) {
            sysUser.setUnitId(unitId);
        }
        CyPersistModel data = sysUserServiceImpl.removeLogical(sysUser);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:delete')")
    @CyOpeLogAnno(title = "system-用户表管理-删除用户表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "删除用户表", notes = "删除用户表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId, SysUser sysUser) {
        sysUser.setBusinessId(businessId);
        CyPersistModel data = sysUserServiceImpl.remove(sysUser);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:update')")
    @CyOpeLogAnno(title = "system-用户表管理-修改用户表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value = "修改用户表", notes = "修改用户表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable int businessId, @Valid SysUser sysUser, BindingResult bindingResult) {
        sysUser.setBusinessId(businessId);
        CyPersistModel data = sysUserServiceImpl.merge(sysUser);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUser);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:query')")
    @CyOpeLogAnno(title = "system-用户表管理-查询用户表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询单一用户表", notes = "查询单一用户表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUserServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:list')")
    @CyOpeLogAnno(title = "system-用户表管理-查询用户表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询用户表集合", notes = "查询用户表集合")
    @GetMapping(value = "/listAll")
    public CyResult listAll(SysUser sysUser) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUserServiceImpl.findAll(sysUser));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:list')")
    @CyOpeLogAnno(title = "system-用户表管理-查询用户表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "分页查询用户表集合", notes = "分页查询用户表集合")
    @GetMapping(value = "/list")
    public CyGridModel listByPagination(SysUser sysUser) {
        sysUserServiceImpl.findAllByPagination(getPaginationUtility(), sysUser);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-用户表管理-查询用户表", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出用户表信息", notes = "导出用户表信息")
    @GetMapping(value = "/export")
    public void outSysUser(HttpServletResponse response, SysUser sysUser, @PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName = "";
        switch (excelId) {
            case "0":
                excelName = "用户表信息.xls";
                break;
            case "1":
                excelName = "用户表信息.xlsx";
                break;
            case "2":
                excelName = "用户表信息.csv";
                break;
        }
        List<SysUser> sysUserList = sysUserServiceImpl.export(sysUser);
        CyEpExcelUtil.exportExcel(sysUserList, "用户表信息", "用户表信息", SysUser.class, excelName, response);
    }

    @CyOpeLogAnno(title = "system-用户表管理-导出代言人信息模版", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出代言人信息模版", notes = "导出代言人信息模版")
    @GetMapping(value = "/exportTemplate")
    public void exportTemplate(HttpServletResponse response) {
        OutputStream os = null;
        Workbook workbook = null;
        try {
            response.setCharacterEncoding("UTF-8");
            response.setHeader("content-Type", "application/vnd.ms-excel");
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("员工发佣明细模板.xlsx", "UTF-8"));
            ExportParams exportParams = new ExportParams("员工发佣明细", "员工发佣明细");
            exportParams.setType(ExcelType.XSSF);
            InputStream resourceAsStream = getClass().getClassLoader().getResourceAsStream(global.getEXCEL_TEMPLATE_LOCATION() + STUDENT_IMPORT_TEMPLATE_NAME);
            //workbook = ExcelExportUtil.exportExcel(exportParams,SysUser.class, new ArrayList<>());
//            String s = global.getEXCEL_TEMPLATE_LOCATION() + STUDENT_IMPORT_TEMPLATE_NAME;
            workbook = WorkbookFactory.create(resourceAsStream);
            os = response.getOutputStream();
            workbook.write(os);
        } catch (Exception e) {
            System.out.println("------------------------打印异常日志--------------------------------");
            System.out.println(e);
            throw new CyServiceException("下载模版失败");
        } finally {
            IOUtils.closeQuietly(workbook);
            IOUtils.closeQuietly(os);
        }
    }

    /**
     * 导入代言人信息表
     */
    @CyOpeLogAnno(
            title = "代言人管理-导入代言人信息表",
            businessType = CyLogTypeEnum.IMPORT
    )
    @ApiOperation(
            value = "导入代言人信息",
            notes = "导入代言人信息"
    )
    @PostMapping("/import")
    public CyResult importData(MultipartFile file, Integer deptId) {
        if (deptId == null) {
            return CyResultGenUtil.builder(new CyPersistModel(0), "导入失败，请选择合适商家信息! ",
                    "导入失败，请选择合适商家信息! ", null);
        }
        //校验文件是否为空
        if (file.isEmpty()) {
            return CyResultGenUtil.builder(new CyPersistModel(0), "导入失败，文件为空",
                    "导入失败，文件为空", null);
        }
        //校验文件是否为excel
        String fileName = file.getOriginalFilename();
        if (fileName != null && !fileName.endsWith(".xls") && !fileName.endsWith(".xlsx")) {
            return CyResultGenUtil.builder(new CyPersistModel(0), "导入失败，文件格式不正确",
                    "导入失败，文件格式不正确", null);
        }
        return CyResultGenUtil.builder(new CyPersistModel(1), "操作成功",
                "网络异常，请稍后重试", sysUserServiceImpl.importData(file, deptId));
    }
}
