package org.rcisoft.business.sysuser.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.cmsbankcard.entity.CmsBankCard;
import org.rcisoft.business.cmsbankcard.service.impl.CmsBankCardServiceImpl;
import org.rcisoft.business.cmspriceclick.dao.CmsPriceClickRepository;
import org.rcisoft.business.cmspriceclick.entity.CmsPriceClick;
import org.rcisoft.business.cmsunitnewvideo.dao.CmsUnitNewVideoRepository;
import org.rcisoft.business.cmsunitnewvideo.entity.CmsUnitNewVideo;
import org.rcisoft.business.sysunit.dao.SysUnitRepository;
import org.rcisoft.business.sysuser.dao.AdminSysUserRepository;
import org.rcisoft.business.sysuser.emuns.AdminUserInfoExceptionEnums;
import org.rcisoft.business.sysuser.entity.AddSysUser;
import org.rcisoft.business.sysuser.entity.SysUser;
import org.rcisoft.business.sysuser.entity.SysUserUnit;
import org.rcisoft.business.sysuser.service.AdminSysUserService;
import org.rcisoft.core.component.CyInitComp;
import org.rcisoft.core.constant.CyDelStaCons;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.result.ResultServiceEnums;
import org.rcisoft.sys.rbac.user.dao.SysUserRbacRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * Created by cy on 2024年6月4日 上午10:19:34.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class SysUserServiceImpl extends ServiceImpl<AdminSysUserRepository, SysUser> implements AdminSysUserService {

    @Autowired
    private PasswordEncoder passwordEncoder;

    @Autowired
    private SysUserRbacRepository sysUserRbacRepository;

    @Autowired
    private CmsBankCardServiceImpl cmsBankCardService;

    @Autowired
    private SysUnitRepository sysUnitRepository;

    @Autowired
    private CyInitComp cyInitComp;

    @Autowired
    private CmsUnitNewVideoRepository cmsUnitNewVideoRepository;

    @Autowired
    private CmsPriceClickRepository cmsPriceClickRepository;


    /**
     * 保存 用户表
     *
     * @param userToAdd
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(AddSysUser userToAdd) {
        SysUser sysUser = baseMapper.checkAddRepeat(userToAdd);
        if (null != sysUser) {
            if (StringUtils.isNotEmpty(sysUser.getUsername())) {
                throw new CyServiceException(AdminUserInfoExceptionEnums.USERNAME_EXISTS);
            }

//            if (StringUtils.isNotEmpty(sysUser.getPhone())) {
//                throw new CyServiceException(AdminUserInfoExceptionEnums.PHONE_EXISTS);
//            }

            if (StringUtils.isNotEmpty(sysUser.getIdNumber())) {
                throw new CyServiceException(AdminUserInfoExceptionEnums.ID_NUMBER_EXISTS);
            }
        }
        userToAdd.setPassword(passwordEncoder.encode(userToAdd.getPassword()));
        userToAdd.setDelFlag(CyDelStaCons.NORMAL.getStatus());
        userToAdd.setUserType("1");
        userToAdd.setFlag("1");
        userToAdd.setIdentity(cyInitComp.getIndividual()[0]);
        int line = baseMapper.insert(userToAdd);
        //配置权限 角色
        if (userToAdd.getRoleList() != null && !userToAdd.getRoleList().isEmpty()) {
            sysUserRbacRepository.insertUserRole(userToAdd.getBusinessId(), userToAdd.getRoleList());
        }
        //配置银行卡信息
        if (userToAdd.getBankCardNum() != null && !userToAdd.getBankCardNum().isEmpty()) {
            CmsBankCard cmsBankCard = new CmsBankCard();
            cmsBankCard.setBankCardNum(userToAdd.getBankCardNum());
            cmsBankCard.setUserId(userToAdd.getBusinessId());
            cmsBankCardService.persist(cmsBankCard);
        }
        //配置商家信息
        if (userToAdd.getDeptId() != null) {
            sysUnitRepository.insertByUnitUser(Integer.valueOf(userToAdd.getDeptId().toString()), userToAdd.getBusinessId());
        } else if (userToAdd.getDeptIdPrepares() != null) {
            //,切割deptIdPrepares 变成list
            ArrayList<String> collect = Arrays.stream(userToAdd.getDeptIdPrepares().split(",")).collect(Collectors.toCollection(ArrayList::new));
            collect.forEach(deptId -> {
                sysUnitRepository.insertByUnitUser(Integer.valueOf(deptId), userToAdd.getBusinessId());
            });
        }
        return new CyPersistModel(line);
    }

    /**
     * 删除 用户表
     *
     * @param sysUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(SysUser sysUser) {
        int line = baseMapper.realDelete(sysUser);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                sysUser.getBusinessId() + "的用户表信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 用户表
     *
     * @param sysUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(SysUser sysUser) {
        //超级管理员
        if (CyUserUtil.getAuthenRoleIds().equals(cyInitComp.admin[0])) {
            sysUser.setDeleted();
            int line = baseMapper.deleteById(sysUser);
            //商家代言人表
            sysUnitRepository.deleteTheUserByUserId(sysUser);
            //推广表
            cmsPriceClickRepository.updateTheUserByUserId(sysUser);
            log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                    sysUser.getBusinessId() + "的用户表信息");
            return new CyPersistModel(line);
        }
        //商家管理员
        if (CyUserUtil.getAuthenRoleIds().equals(cyInitComp.unit[0])) {
            //代言人表
            SysUserUnit sysUserUnit = new SysUserUnit();
            sysUserUnit.setUserId(sysUser.getBusinessId());
            sysUserUnit.setUnitId(Integer.valueOf(sysUser.getUnitId()));
            sysUnitRepository.deleteByUnitIdAndUserId(sysUserUnit);
            //推广表
            cmsPriceClickRepository.updateTheUserByMerchantIdUserId(sysUser, sysUser.getUnitId());
        }
        return new CyPersistModel(1);
    }

    /**
     * 修改 用户表
     *
     * @param sysUser
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(SysUser sysUser) {
        //根据新账号名称查询


        //根据用户id查询数据
        SysUser oldUser = baseMapper.selectAllUserInfo(sysUser);
        //修改银行卡账号
        if (StringUtils.isNotEmpty(sysUser.getBankCardNum()) && !oldUser.getBankCardNum().equals(sysUser.getBankCardNum())) {
            CmsBankCard cmsBankCard = new CmsBankCard();
            cmsBankCard.setUserId(sysUser.getBusinessId());
            cmsBankCard.setBankCardNum(sysUser.getBankCardNum());
            cmsBankCardService.merge(cmsBankCard);
        }
        //配置商家信息
        //查询当前用户的商家信息

//        if (sysUser.getDeptId() != null && sysUser.getDeptIdPrepare() != null && !sysUser.getDeptId().equals(sysUser.getDeptIdPrepare())) {
//            List<SysUserUnit> userUnits = baseMapper.selectByUnitIdAndUserId(sysUser.getBusinessId(), sysUser.getDeptIdPrepare());
//            if (CollectionUtils.isNotEmpty(userUnits)) {
//                //删除查询到的数据,并且添加
//                sysUnitRepository.deleteByUnitIdAndUserId(userUnits.get(0));
//            }
//            sysUnitRepository.insertByUnitUser(Integer.valueOf(sysUser.getDeptId().toString()), sysUser.getBusinessId());
//        }
        List<Integer> userUnitIds = baseMapper.selectByUnitIds(sysUser.getBusinessId());
        //1.当我与现在的多时 //需要将我传进来 与 我查出来的比较 看我现在多那些id
        List<String> list = Stream.of(sysUser.getDeptIdPrepares().split(","))
                .collect(Collectors.toList());
        // 将 userUnitIds 转换为字符串列表
        List<String> userUnitIdStrings = userUnitIds.stream()
                .map(Object::toString)
                .collect(Collectors.toList());

        // 保留与 userUnitIds 不相交的元素
        List<String> now = list.stream()
                .filter(element -> !userUnitIdStrings.contains(element))
                .collect(Collectors.toList());
        for (String s : now) {
            //1 将关系绑定
            sysUnitRepository.insertByUnitUser(Integer.valueOf(s), sysUser.getBusinessId());
          /*  //查询出当前商家下的文章以及视频信息
//            List<CmsUnitNewVideo> cmsUnitNewVideos = cmsUnitNewVideoRepository.queryVideoOrArticleRelationships(Integer.valueOf(s));
            //循环添加当前文章或视频
            for (CmsUnitNewVideo cmsUnitNewVideo : cmsUnitNewVideos) {
                CmsPriceClick cmsPriceClick = new CmsPriceClick();
                cmsPriceClick.setType(cmsUnitNewVideo.getType());
                cmsPriceClick.setNewsVideoId(cmsUnitNewVideo.getNewsVideoId());
                cmsPriceClick.setUserId(sysUser.getBusinessId());
                cmsPriceClick.setUnitId(Integer.valueOf(s));
                cmsPriceClick.setPrice(BigDecimal.valueOf(0));
                cmsPriceClick.setDelFlag(String.valueOf(0));
                cmsPriceClick.setFlag(String.valueOf(1));
                cmsPriceClick.setClickThroughRate(0);
                cmsPriceClick.setCreateBy(CyUserUtil.getAuthenBusinessId());
                cmsPriceClick.setUpdateBy(CyUserUtil.getAuthenBusinessId());
                cmsPriceClick.setCreateDate(new Date());
                cmsPriceClick.setUpdateDate(new Date());
                cmsPriceClickRepository.insert(cmsPriceClick);
            }*/

        }
        //2 userUnitIdStrings保留与list不相交的元素
        List<String> old = userUnitIdStrings.stream()
                .filter(element -> !list.contains(element))
                .collect(Collectors.toList());
        for (String s : old) {
            //接触关系 以及物理删除和逻辑删除关联表和业务信息
            //1.代言人与商家表
            SysUserUnit sysUserUnit = new SysUserUnit();
            sysUserUnit.setUnitId(Integer.valueOf(s));
            sysUserUnit.setUserId(sysUser.getBusinessId());
            sysUnitRepository.deleteByUnitIdAndUserId(sysUserUnit);
            //2.新闻视频推广次数价格表
            cmsPriceClickRepository.updateTheUserByMerchantIdUserId(sysUser, s);
        }
        //3.修改用户信息
        sysUser.setPassword(null);
        sysUser.setFlag(null);
        int line = baseMapper.updateById(sysUser);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + sysUser.getBusinessId() + "的用户表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 用户表
     *
     * @param id
     * @return
     */
    @Override
    public SysUser findById(int id) {
        return baseMapper.selectById(id);
    }

    /**
     * 分页查询 用户表
     *
     * @param sysUser
     * @return
     */
    @Override
    public IPage<SysUser> findAllByPagination(CyPageInfo<SysUser> paginationUtility,
                                              SysUser sysUser) {
        //判断当前用户角色 超级管理员还是 商家管理员
        if (CyUserUtil.getAuthenRoleIds().equals(cyInitComp.admin[0])) {
            IPage<SysUser> sysUserIPage = baseMapper.querySysUsersPaged(paginationUtility, sysUser);
            //配置将deptIdPrepare 赋值 给 deptId
            for (SysUser record : sysUserIPage.getRecords()) {
                record.setDeptId(record.getDeptIdPrepare());
            }
            return sysUserIPage;
        }
        if (CyUserUtil.getAuthenRoleIds().equals(cyInitComp.unit[0])) {
            if (sysUser.getDeptId() != null) {
                IPage<SysUser> sysUserIPage = baseMapper.querySysUsersPaged(paginationUtility, sysUser);
                //配置将deptIdPrepare 赋值 给 deptId
                for (SysUser record : sysUserIPage.getRecords()) {
                    record.setDeptId(record.getDeptIdPrepare());
                }
                return sysUserIPage;
            }
        }

        IPage<SysUser> sysUserIPage = new Page<>();
        return sysUserIPage;
    }

    /**
     * 查询list 用户表
     *
     * @param sysUser
     * @return
     */
    @Override
    public List<SysUser> findAll(SysUser sysUser) {
        return baseMapper.querySysUsers(sysUser);

    }

    /**
     * 导出用户表
     *
     * @return
     */
    @Override
    public List<SysUser> export(SysUser sysUser) {
        List<SysUser> sysUserList = baseMapper.querySysUsers(sysUser);
        return sysUserList;
    }

    /**
     * 导入代言人信息
     *
     * @param file
     * @return
     */
    @Override
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    public String importData(MultipartFile file, Integer deptId) {
        List<AddSysUser> users = CyEpExcelUtil.importExcel(file, 0, 1, AddSysUser.class);
        //正确的条数
        int successNum = 0;
        //错误的条数
        int errorNum = 0;
        // 初始化行号
        int rowIndex = 1;

        StringBuilder errorMessage = new StringBuilder();
        if (CollectionUtils.isNotEmpty(users)) {
            rowIndex += 1;
            //导入信息校验
            //剔除对象中的空对象
            ArrayList<AddSysUser> list = new ArrayList<>();
            Pattern pattern = Pattern.compile("[0-9]*");
            for (AddSysUser user : users) {
                if (StringUtils.isNotEmpty(user.getName()) && StringUtils.isNotEmpty(user.getPhone()) && StringUtils.isNotEmpty(user.getIdNumber())) {
                    //身份证信息判断 长度判断
                    if (StringUtils.isEmpty(user.getIdNumber()) || user.getIdNumber().length() != 18) {
                        throw new CyServiceException(ResultServiceEnums.UPLOAD_ERROR.getCode(),
                                "第" + rowIndex + "行，身份证信息（" + user.getIdNumber() + "）不能为空,且长度为18位。请校验数据并重新导入! ");
                    }
                    //手机号
                    //数字长度对比
                    if (StringUtils.isEmpty(user.getPhone()) || user.getPhone().length() != 11) {
                        throw new CyServiceException(ResultServiceEnums.UPLOAD_ERROR.getCode(),
                                "第" + rowIndex + "行，手机号（" + user.getPhone() + "）不能为空,且长度为11位。请校验数据并重新导入! ");
                    }
                    //银行卡号
                    if (StringUtils.isEmpty(user.getBankCardNum()) || user.getBankCardNum().length() > 20 || user.getBankCardNum().length() < 15) {
                        throw new CyServiceException(ResultServiceEnums.UPLOAD_ERROR.getCode(),
                                "第" + rowIndex + "行，银行卡号（" + user.getBankCardNum() + "）不能为空,长度不小于15位且不大于20位。请校验数据并重新导入! ");
                    }
                    //醒目长度
                    if (StringUtils.isEmpty(user.getName()) || user.getName().length() > 6 || user.getName().length() < 2) {
                        throw new CyServiceException(ResultServiceEnums.UPLOAD_ERROR.getCode(),
                                "第" + rowIndex + "行，姓名（" + user.getName() + "）不能为空,长度不小于2位且不大于6位。请校验数据并重新导入! ");
                    }
                    //手机号只能是数字
                    if (!pattern.matcher(user.getPhone()).matches()) {
                        throw new CyServiceException(AdminUserInfoExceptionEnums.PHONE_LENGTH_ERROR.getCode(), AdminUserInfoExceptionEnums.PHONE_LENGTH_ERROR.getMessage());
                    }
                    //银行卡号只能是数字
                    if (!pattern.matcher(user.getBankCardNum()).matches()) {
                        throw new CyServiceException(AdminUserInfoExceptionEnums.BANK_LENGTH_ERROR.getCode(), AdminUserInfoExceptionEnums.BANK_LENGTH_ERROR.getMessage());
                    }
                    list.add(user);
                }

            }
            if (CollectionUtils.isEmpty(list)) {
                throw new CyServiceException(ResultServiceEnums.UPLOAD_ERROR.getCode(), "导入文件为空,请校验导入文档数据!");
            }
            //根据用户信息查询是否包含相同用户,如果用户信息相同,则不生成账号信息,添加对应的单位信息
            for (AddSysUser user : list) {
                if (StringUtils.isNotEmpty(user.getName()) && StringUtils.isNotEmpty(user.getPhone()) && StringUtils.isNotEmpty(user.getIdNumber())) {
                    //校验上传对象 用户是否存在 只根据身份证信息判断
                    SysUser sysUser = baseMapper.checkAddUpload(user);
                    //如果用户存在且这idNumber 一致则不生成新的账号
                    if (sysUser == null) {
                        //如果用户不存在,则生成账号
                        user.setUsername(user.getIdNumber());
                        //密码为身份证号码后六位
                        user.setPassword(user.getIdNumber().substring(user.getIdNumber().length() - 6));
                        //商家
                        user.setDeptId(BigInteger.valueOf(deptId));
                        //角色为代言人
                        ArrayList<Integer> integers = new ArrayList<>();
                        integers.add(3);
                        user.setRoleList(integers);
                        persist(user);
                        //如果当前商家拥有视频或者文章 需要新增 代言人绑定视频或者文章
//                        spokespersonConfigure(deptId, user);
                        successNum++;
                    }
                    //不添加用户信息 角色等
                    if (sysUser != null) {
                        //配置商家信息 商家ids
                        List<Integer> ids = baseMapper.selectByUnitIds(sysUser.getBusinessId());
                        //当前的 这个代言人没有配置商家信息
                        if (CollectionUtils.isEmpty(ids)) {
                            sysUnitRepository.insertByUnitUser(deptId, sysUser.getBusinessId());
                            //因为当前这个代言人还没有配置过代言人信息，所以当前商户的代言是直接添加
//                            spokespersonConfigure(deptId, sysUser);
                        }
                        //当前代言人配置过商家信息，但是不包含当前商家信息
                        if (CollectionUtils.isNotEmpty(ids) && !ids.contains(deptId)) {
                            sysUnitRepository.insertByUnitUser(deptId, sysUser.getBusinessId());
//                            spokespersonConfigure(deptId, sysUser);
                        }
                        successNum++;
                    } else if (sysUser != null && (sysUser.getName() == null || sysUser.getPhone() == null || sysUser.getIdNumber() == null)) {
                        //如果不一致,则记录错误信息
                        errorMessage.append("第" + (errorNum + 1) + "条数据导入失败,请检查数据是否正确,错误信息:"
                                + user.getName() + ":" + user.getPhone() + ":" + user.getIdNumber());
                    }
                }
            }
        }
        if (successNum > 0) {
            return successNum + "条数据导入成功，" + errorMessage.toString();
        } else {
            throw new CyServiceException("导入失败" + errorMessage);
        }
    }

    /**
     * 如果当前商家拥有视频或者文章 需要新增 代言人绑定视频或者文章
     *
     * @param deptId
     * @param sysUser
     */
    private void spokespersonConfigure(Integer deptId, SysUser sysUser) {
        List<CmsUnitNewVideo> cmsUnitNewVideos = cmsUnitNewVideoRepository.queryVideoOrArticleRelationships(deptId);
        if (CollectionUtils.isNotEmpty(cmsUnitNewVideos)) {
            for (CmsUnitNewVideo cmsUnitNewVideo : cmsUnitNewVideos) {
                CmsPriceClick cmsPriceClick = new CmsPriceClick();
                cmsPriceClick.setNewsVideoId(cmsUnitNewVideo.getNewsVideoId());
                cmsPriceClick.setUnitId(Integer.valueOf(cmsUnitNewVideo.getUnitId()));
                cmsPriceClick.setUserId(sysUser.getBusinessId());
                cmsPriceClick.setType(cmsUnitNewVideo.getType());
                cmsPriceClick.setFlag(String.valueOf(1));
                cmsPriceClick.setClickThroughRate(0);
                cmsPriceClick.setPrice(BigDecimal.valueOf(0));
                cmsPriceClickRepository.insert(cmsPriceClick);
            }
        }
    }


    public ArrayList<SysUser> selectIdcardBid() {
        return baseMapper.selectIdcardBid();
    }
}

